/*
 *
 * Copyright (C) 2002 Zsolt Rizsanyi <rizsanyi@myrealbox.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef HAVE_XDPMS
# define QT_CLEAN_NAMESPACE
#include <kdebug.h>
# include <qwindowdefs.h>
# include <X11/Xlib.h>
extern "C" {
# include <X11/extensions/dpms.h>
}
#endif

#include "dpmscontrol.h"

bool DpmsControl::getDpmsStatus()
{
#ifdef HAVE_XDPMS
    Display *displ = qt_xdisplay();
    int nothing;
    if (DPMSQueryExtension(displ, &nothing, &nothing) && DPMSCapable(displ)) {
	CARD16 power_level;
	BOOL state;
	if (DPMSInfo(displ, &power_level, &state)) {
	    return state;
	}
    }
#endif
    return false;
}

bool DpmsControl::enableDpms(bool enable)
{
#ifdef HAVE_XDPMS
    Display *displ = qt_xdisplay();
    int nothing;
    if (DPMSQueryExtension(displ, &nothing, &nothing) && DPMSCapable(displ)) {
	Status ret;
	if (enable)
	    ret = DPMSEnable(displ);
	else
	    ret = DPMSDisable(displ);
	if (ret) {
	    CARD16 power_level;
	    BOOL state;
	    if (DPMSInfo(displ, &power_level, &state)) {
		return enable == state;
	    }
	} else {
	    kdDebug() << "dpms not available?!" << endl;
	    return false;
	}
    }
#else
    kdDebug() << "Trying to "<< (enable?"enable":"disable") <<
	" dpms, but not compiled in" << endl;
#endif
    return false;
}

bool DpmsControl::disableDpms()
{
    return DpmsControl::enableDpms(false);
}
