/*
 *
 * Copyright (C) 2002 Zsolt Rizsanyi <rizsanyi@myrealbox.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __DPMS_CONTROL_H__
#define __DPMS_CONTROL_H__

/**
 * DPMS Control class
 *
 * You can call the static methods of this class directly
 * from within another class to control the DPMS or
 * to get its current status.
 *
 * @version $Id: dpmscontrol.h,v 1.1 2002/09/23 22:36:30 staikos Exp $
 */
class DpmsControl
{
public:
    /**
     * Static accessor method to get the status of the dpms.
     * Returns true if DPMS is currently enabled,
     * false if it is disabled.
     */
    static bool getDpmsStatus();

    /**
     * Static accessor method to enable or disable DPMS. Returns
     * true if the operation was successful, false if it was not.
     */
    static bool enableDpms(bool enable = true);

    /**
     * Disables DPMS. Calls enableDpms(false).
     */
    static bool disableDpms();

private:
    bool        dpmsWasEnabled;
};

#endif
