// -*- c++ -*-

/*
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <kmainwindow.h>
#include <kaction.h>

class QSplitter;
class QVBox;

class KAction;
class KPopupMenu;
class KConfig;

class QtVisionWidget;
class QtVisionActions;
class QtVisionSysTray;

class Sidebar;
class ToggleViewAction;

class Lirc;

/**
 * MainWindow shell for QtVision.
 *
 * @version $Id: mainwindow.h,v 1.27 2003/01/31 02:16:30 staikos Exp $
 */
class MainWindow : public KMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget* parent = 0, const char* name = 0, WFlags f = 0);
    ~MainWindow();

    KPopupMenu *createMenu( const QString &menuname );
    KPopupMenu *insertContextMenu( const QString &menuname );

public slots:
    void toggleMenubar();
    void toggleToolbar();
    void toggleStatus();
    void disableScreenSaver( bool );
    void stayOnTopCfgChanged( bool );
    void screenSaverCfgChanged( bool, int );
    void screenSaverModeChanged( int );
    void screenSaverVisible( bool );

    void showChangeLog();
    void showCVSLog();
    void showWebSite();

    /**
     * Activates/deactivates sleep timer.
     */
    virtual void sleepTimer();

signals:
    /// Emits if a lirc number key was pressed
    void numberKeyPressed(int);

    /**
     * Emitted when the "time to sleep before turning off" has been changed.
     */
    void sleepTimerDelay(const QString &);

protected slots:
    void setShowTrayIcon( bool show );
    void checkViewMode(int);
    void lircCommand(const QString &, unsigned);
    void keyBindings();
    void sbProgramName(const QString& name);
    void statusMessage(const QString& name);
    void shortMessage(const QString& name);
    void clearStatusBar();

protected:
    void createActions();
    void setupInfraRed();

    void saveWindowSize( KConfig * );
    void restoreWindowSize( KConfig * );

private slots:
    void slotSleepTimerTimeout();
    
private:
    QtVisionWidget* _screen;
    QtVisionActions* _actions;
    QtVisionSysTray* _systray;
    
    QTimer *_sleeptimer;
    QSplitter* _view;
    Sidebar* _side;
    
    KAction* _actChangeLog;
    KAction* _actCVSLog;
    KAction* _actWebSite;
    
    KToggleAction* _sleepTime;
    
    ToggleViewAction* _actShowSidebar;
    ToggleViewAction* _actShowViewbar;
    ToggleViewAction* _actShowVolbar;
    Lirc* _lirc;
    
    bool _screenSaverWasEnabled;
    bool _dpmsWasEnabled;
    bool _activateVisibility;
        
    int _sleepdelay;
};

#endif // MAINWINDOW_H

