// -*- c++ -*-

/*
 *
 * Copyright (C) 2002 Richard Moore, <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef QTVISIONACTIONS_H
#define QTVISIONACTIONS_H

#include <qobject.h>

class KAction;
class KRadioAction;

class LCDNumberAction;
class SliderAction;

class QtVision;
class QtVisionWidget;
class QVSourcePlugin;

/**
 * Creates and manages the KActions for a QtVision object.
 *
 * @version $Id: qtvisionactions.h,v 1.20 2002/10/17 14:18:05 staikos Exp $
 * @author Richard Moore, rich@kde.org
 */
class QtVisionActions : public QObject
{
    Q_OBJECT

public:
    QtVisionActions( QtVision *qtv, const char *name=0 );
    ~QtVisionActions();

    void createActions( QtVisionWidget *qtvw, KActionCollection *acts );

    KRadioAction *showNormal() const { return showNorm; }
    KRadioAction *showFullScreen() const { return showFull; }
    KRadioAction *showTopLevel() const { return showTop; }
    KRadioAction *showVideoDesktop() const { return showDesk; }
    KAction *nextViewMode() const { return nextView; }
    KAction *toggleFullScreen() const { return toggleFull; }

    void createActions( KActionCollection *acts );

    KAction *channelDown() const { return chanDown; }
    KAction *channelUp() const { return chanUp; }
    KAction *channelWizard() const { return chanWiz; }
    KAction *channelLoadDefaults() const { return chanLoadDefaults; }
    KAction *channelImportLegacy() const { return chanImportLegacy; }
    KAction *channelOpen() const { return chanOpen; }
    KAction *channelImport() const { return chanImport; }

    SliderAction *volumeSlider() const { return volSlider; }
    KAction *volumeMute() const { return volMute; }
    KAction *volumeUp() const { return volUp; }
    KAction *volumeDown() const { return volDown; }

    KAction *snapshot() const { return snap; }
    KAction *changeSrc() const { return chngSrc; }
    KAction *settings() const { return showSettings; }
    KAction *keys() const { return showKeys; }

    LCDNumberAction *channel() const { return chan; }

public slots:
    void deviceChanged( QVSourcePlugin *dev );
    void updateSlider(int, int);
    void updateMuteButton(bool);
    void updateViewMode( int mode );

signals:
    void launchBindings();

private:
    QtVision *qtv;

    LCDNumberAction *chan;
    KAction *chanLoadDefaults;
    KAction *chanImportLegacy;
    KAction *chanOpen;
    KAction *chanImport;
    KAction *chanDown;
    KAction *chanUp;
    KAction *chanWiz;

    SliderAction *volSlider;
    KToggleAction *volMute;
    KToggleAction *sleepTime;
    KAction *volUp;
    KAction *volDown;
    KAction *prevChannel;

    KRadioAction *showNorm;
    KRadioAction *showFull;
    KRadioAction *showTop;
    KRadioAction *showDesk;
    KAction *nextView;
    KAction *toggleFull;

    KAction *snap;
    KAction *chngSrc;
    KAction *showSettings;
    KAction *showKeys;
};

#endif // QTVISIONACTIONS_H
