/*
 *
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <qlayout.h>
#include <qpopupmenu.h>
#include <qtimer.h>
#include <qvbox.h>

#include <kdebug.h>
#include <kglobal.h>
#include <klibloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>

#include "qtvision.h"
#include "qtvisionpart.h"
#include "qtvisionwidget.h"

#include "qtvisionapplet.h"
#include "qtvisionapplet.moc"

static const int QTV_APPLET_FLAGS = KPanelApplet::Preferences | KPanelApplet::Help
                                  | KPanelApplet::About | KPanelApplet::ReportBug;

// Factory function
extern "C" KPanelApplet *init( QWidget *parent, const QString& configFile )
{
    KGlobal::locale()->insertCatalogue("qtvisionapplet");

    KLibLoader::self()->globalLibrary("libqtvision.so");
    return new QtVisionApplet( configFile,
			       KPanelApplet::Normal, QTV_APPLET_FLAGS,
			       parent, "qtvisionapplet" );
}


QtVisionApplet::QtVisionApplet( const QString &configFile,
				Type t, int actions,
				QWidget *parent, const char *name )
    : KPanelApplet( configFile, t, actions, parent, name )
{
    QVBoxLayout *box =  new QVBoxLayout( this );
    tv = new QtVisionPart( parent, 0, 0, 0, QStringList() );
    box->addWidget( tv->widget() );

    updateGeometry();
}

QtVisionApplet::~QtVisionApplet()
{

}

int QtVisionApplet::widthForHeight( int h ) const
{
//    return (4*h)/3;
    return h;
}

int QtVisionApplet::heightForWidth( int w ) const
{
//    return (3*w)/4;
    return w;
}

void QtVisionApplet::about()
{
    tv->about();
}

void QtVisionApplet::help()
{
    tv->help();
}

void QtVisionApplet::preferences()
{
    tv->preferences();
}

void QtVisionApplet::reportBug()
{
    tv->reportBug();
}


