/*
 *
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <qlayout.h>

#include <kglobal.h>
#include <klibloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>

#include "qtvision.h"
#include "qtvisionpart.h"
#include "qtvisionwidget.h"

#include "qtvisionextension.h"
#include "qtvisionextension.moc"

extern "C" KPanelExtension *init( QWidget *parent, const QString& configFile )
{
    KGlobal::locale()->insertCatalogue("qtvisionextension");
    KLibLoader::self()->globalLibrary("libqtvision.so");

    return new QtVisionExtension( configFile, KPanelExtension::Normal,
				  KPanelExtension::Preferences
				  |KPanelExtension::Help
				  |KPanelExtension::About
				  |KPanelExtension::ReportBug,
				  parent, "qtvisionextension" );
}

QtVisionExtension::QtVisionExtension( const QString &configFile, Type t, int actions,
				QWidget *parent, const char *name )
    : KPanelExtension( configFile, t, actions, parent, name )
{
    tv = new QtVisionPart( this, 0, 0, 0, QStringList() );
    screen = tv->widget();
    screen->view()->setFixedSize( 240, 240 );
    screen->setFixedSize( 240, 240 );
}

QtVisionExtension::~QtVisionExtension()
{

}

void QtVisionExtension::showEvent( QShowEvent *se )
{
//    resize( screen->size() );
    updateGeometry();
    KPanelExtension::showEvent( se );
    setFixedSize( 240, 240 );
    resize( 240, 240 );
}

void QtVisionExtension::positionChange( Position /* position */)
{
}


QSize QtVisionExtension::sizeHint( Position , QSize max ) const
{
    Q_UNUSED(max);
//    return screen->size();
//    int sz = (max.width() < max.height()) ? max.width() : max.height() ;
    return QSize( 240, 240 );
}

void QtVisionExtension::about()
{
    tv->about();
}

void QtVisionExtension::help()
{
    tv->help();
}

void QtVisionExtension::preferences()
{
    tv->preferences();
}

void QtVisionExtension::reportBug()
{
    tv->reportBug();
}


