// -*- c++ -*-

/*
 *
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef QTVISIONPART_H
#define QTVISIONPART_H

#include <kparts/part.h>

class KPopupMenu;

class QtVisionWidget;
class QtVisionActions;

/**
 * KParts wrapper for a QtVision widget.
 *
 * @author Richard Moore, <rich@kde.org>
 * @version $Id: qtvisionpart.h,v 1.4 2002/07/07 02:02:57 rich Exp $
 */
class QtVisionPart : public KParts::ReadOnlyPart
{
    Q_OBJECT

public:
    QtVisionPart( QWidget *wparent, const char *wname,
		  QObject *parent, const char *name,
		  const QStringList &args );
    virtual ~QtVisionPart();

    /**
     * Returns the name of the file shown in the part.
     */
    QString filename() const { return m_file; }

    /**
     * Returns the QtVisionWidget of this part.
     */
    virtual QtVisionWidget *widget() const { return tv; }

    /**
     * Adds the standard context menu.
     */
    void addContextMenu( bool xmlgui = true );

    /**
     * Creates the standard screen context menu. If the  menu cannot be
     * loaded (or xmlgui is false) the default menu is created.
     */
    KPopupMenu *createContextMenu( bool xmlgui = true );

    /**
     * Creates the named context menu using XMLGUI.
     */
    KPopupMenu *createContextMenu( const QString &name );

    static KAboutData *createAboutData();

public slots:
    virtual void about();
    virtual void help();
    virtual void preferences();
    virtual void reportBug();

    void setAutoStart( bool enable ) { autoStart = enable; }
    void start();

protected:
    virtual bool openFile();

protected slots:
    void maybeStart();

private:
    QtVisionWidget *tv;
    QtVisionActions *actions;
    bool autoStart;
};

#endif // QTVISIONPART_H


// Local Variables:
// c-basic-offset: 4
// End:
