/***************************************************************************
                          qtvisionsystray.cpp  -  A system tray icon that
                          is displayed when the VideoDesktop is enabled
                             -------------------
    begin                : Sun Jul 21 2002
    copyright            : (C) 2002 by Kevin Hessels
    email                : khessels@shaw.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kaction.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kstdaction.h>
#include <ksystemtray.h>


#include <qtooltip.h>

#include "actions.h"
#include "mainwindow.h"
#include "qtvisionactions.h"
#include "qtvisionsystray.h"
#include "qtvisionsystray.moc"

QtVisionSysTray::QtVisionSysTray(MainWindow *parent, const char *name, QtVisionActions *a)
				: KSystemTray(parent, name), actions(a), mainWin(parent)
{
	setPixmap(BarIcon("qtvision"));
        menu = mainWin->createMenu( "system_tray_popup" );
	QToolTip::add(this, i18n("QtVision Control"));
	setAcceptDrops(false);
	return;
} // QtVisionSysTray constructor


QtVisionSysTray::~QtVisionSysTray()
{
	return;
} // QtVisionSysTray Destructor


/***************************************************************************
*
* Protected Functions
*
***************************************************************************/

void QtVisionSysTray::mouseDoubleClickEvent(QMouseEvent *)
{
	emit doubleClicked();
	return;
} // mouseDoubleClickEvent


void QtVisionSysTray::mousePressEvent(QMouseEvent *ev)
{
	switch (ev->button()) {
		case RightButton:
			ev->accept();
			if ( menu )
			    menu->popup(ev->globalPos());
			break;
		default:
			ev->ignore();
			emit leftClicked();
			break;
	} // switch
	
	return;
} // mousePressEvent


void QtVisionSysTray::wheelEvent(QWheelEvent *ev)
{
	ev->accept();

	if (ev->delta() > 0)
		(void) actions->channelUp()->activate();
	else if (ev->delta() < 0)
		(void) actions->channelDown()->activate();

	return;
} // wheelEvent
