/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "miscmanager.h"

#include "qvmiscplugin.h"
#include "qtvision.h"
#include <kdebug.h>


MiscManager::MiscManager(QtVision *qtv, QWidget *w) {
	_view = w;
	_qtv = qtv;
	scanPlugins();
}

MiscManager::~MiscManager() {
}


void MiscManager::rescanPlugins() {
	// scanPlugins will remove turned off plugins and load the newly turned on ones
	scanPlugins();
}

void MiscManager::scanPlugins() {
	QPtrList<PluginDesc>& miscPlugins(_qtv->pluginFactory()->miscPlugins());

	for (PluginDesc *plug = miscPlugins.first(); plug;
			plug = miscPlugins.next()) {
		bool skip = false;
		for (QVMiscPlugin *p = _plugs.first(); p; p = _plugs.next()) {
			if (p->pluginDescription() == *plug) {
				if (plug->enabled)
					p->updateDescription(*plug);
				else
				{
					_plugs.remove(p);
					delete p;
				}
				skip = true;
				break;
			}
		}

		if (skip)
			continue;

		kdDebug() << "Found a plugin:" << endl;
		kdDebug() << plug->name << endl;
		kdDebug() << plug->author << endl;
		kdDebug() << plug->comment << endl;
		if (plug->enabled) {
			QVMiscPlugin *p = _qtv->pluginFactory()->getMiscPlugin(plug,_view);
			if (p)
				_plugs.append(p);
		} else
			kdDebug() << "The plugin is disabled" << endl;
	}

}

QPtrList<QVMiscPlugin> MiscManager::plugins() {
return _plugs;
}

bool MiscManager::filterNumberKey(int key) {
    for (QVMiscPlugin *p = _plugs.first(); p; p = _plugs.next())
        if (p->filterNumberKey(key))
            return true;
    return false;
}

void MiscManager::installGUIElements(KXMLGUIFactory *guiFactory,
		KActionCollection *actionCollection)
{
    for (QVMiscPlugin *p = _plugs.first(); p; p = _plugs.next())
		p->installGUIElements(guiFactory, actionCollection);
}

#include "miscmanager.moc"

// vim: sw=4 ts=4
