/* 
   Copyright (c) 2002 Malte Starostik <malte@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// $Id: telex.h,v 1.6 2003/07/27 15:40:20 rizsanyi Exp $

#ifndef QVSIMON_PLUGIN_H
#define QVSIMON_PLUGIN_H

#include <qguardedptr.h>

#include <kurl.h>

#include <qvmiscplugin.h>
#include "telexiface.h"

class QCheckBox;
class QComboBox;

struct vbi_decoder;

namespace Telex
{
    class Display;
    class Worker;
    class Page
    {
    public:
        Page();
        Page( vbi_decoder* decoder, int page, int sub = -1 );
        Page( const Page& other )
            : m_data( 0 ) { *this = other; }
        ~Page() { release(); }
        Page& operator =( const Page& );

        operator const void*() const { return m_decoder; }
        bool operator ==( const Page& rhs ) const;
        bool operator !=( const Page& rhs ) const
            { return !( *this == rhs ); }

        int number() const { return m_number; }
        int sub() const { return m_sub; }

        void refresh();
        QPixmap header( int width, int height, int current = -1, int request = -1 ) const;
        QPixmap render( int width, int height, int request = -1 ) const;

        bool available() const;
        int columns() const;
        int rows() const;
        bool isLink( int, int ) const;
        class Link
        {
            // can't use a simple struct because of the circular dependency
            // Page <=> Page::Link
        public:
            Link() : m_type( Null ), m_data( 0 ) {}
            Link( const Page& p );
            Link( const KURL& u );
            ~Link();

            enum Type { Null, TTX, URL };
            Type type() const { return m_type; }
            const Page& page() const;
            const KURL& url() const;

        private:
            Type m_type;
            struct Data;
            Data* m_data;
        };
        Link link( int, int ) const;

        static int pageOffset;

    private:
        enum Validity { None = -1, Header, All };
        void release() const;
        bool require( Validity, int = -1, int = -1 ) const;

        mutable vbi_decoder* m_decoder;
        int m_number;
        int m_sub;
        struct Data;
        mutable Data* m_data;
    };

    class Plugin : public QVMiscPlugin, virtual public TelexIface
    {
        Q_OBJECT
    public:
        Plugin( QtVision *qtv, QWidget* parent );
        virtual ~Plugin();

        virtual bool filterNumberKey( int );

        /** Install toolbar button to (un)hide teletext */
        virtual void installGUIElements(KXMLGUIFactory * guiFactory,
                KActionCollection *actionCollection);

    public slots:
        virtual QWidget* configWidget( QWidget* parent, const char* name );
        virtual void saveConfig();

        void showDisplay(bool);
        void toggleShow();

        // navigate through the pages
        void navigate(int page);
        void nextPage();
        void previousPage();

    private slots:
        void channelChanged();
        void navigate( const Page::Link& );

    protected:
        virtual void customEvent( QCustomEvent* );

    private:
        QGuardedPtr< Display > m_display;
        Worker* m_worker;

        Page m_page;
        int m_pageInput;
    };

    class Display : public QWidget
    {
        Q_OBJECT
    public:
        Display( QWidget* parent );
        virtual ~Display();

        virtual bool eventFilter( QObject*, QEvent* );

    public slots:
        void setTransparent( bool );
        void setHeader( const Page& );
        void setPage( const Page& );
        void setRequest( int );
        void refresh();

    protected:
        virtual void updateMask();
        virtual void resizeEvent( QResizeEvent* );
        virtual void paintEvent( QPaintEvent* );
        virtual void mouseMoveEvent( QMouseEvent* );
        virtual void mousePressEvent( QMouseEvent* );

    signals:
        void navigate( const Page::Link& );

    private:
        void updatePage();

        Page m_page;
        QPixmap m_scaled;
        bool m_transparent;
        int m_request;
    };
}

#endif

// vim: ts=4 sw=4 et
