// -*- c++ -*-

/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */



#ifndef __QVV4L_PLUGIN_H
#define __QVV4L_PLUGIN_H

#include <qvsrcplugin.h>
#include <qmutex.h>
#include <qmap.h>
#include <qtimer.h>

#include "v4lplugincfg.h"

#include "qvideostream.h"

class V4LDev;
class V4LGrabber;
class Clipper;

/**
 * Video source plugin for Video4Linux.
 */
class QtVisionV4L : public QVSourcePlugin
{
    Q_OBJECT

public:
    QtVisionV4L(QtVision *qtv, QWidget *parent = 0, const char* name = 0);
    virtual ~QtVisionV4L();

    /**
     * To get various values.
     */
    virtual int brightness();
    virtual int colour();
    virtual int hue();
    virtual int contrast();
    virtual int whiteness();
    virtual int frequency();
    virtual bool muted();
    virtual int tunerMode();
    virtual bool isTuner();
    virtual int signal();
    virtual QColor colourKey();

    virtual int probeDevices();

    virtual bool videoPlaying() const;

    virtual bool event(QEvent *e);

    virtual int norm() const;

public slots:
    /**
     * Sets the device to use.
     */
    virtual int setDevice( const QString &dev );

    /**
     * Sets the source to use.
     */
    virtual int setSource( const QString &src );

    /**
     * Sets the tuner mode (PAL, NTSC etc.).
     */
    virtual void setNorm( int n );

    /**
     * Sets the current current channel.
     */
    virtual void setFrequency( int freq );

    /**
     * Mutes the sound.
     */
    virtual void setMuted( bool muted );

    virtual void setBrightness(int val);
    virtual void setColour(int val);
    virtual void setHue(int val);
    virtual void setContrast(int val);
    virtual void setWhiteness(int val);

    virtual void viewMoved(int x, int y);
    virtual void viewResized(int w, int h);

    virtual int startVideo();
    virtual int stopVideo();
    
    virtual int setVideoDesktop(bool on);
    virtual bool canVideoDesktop() const;

    virtual bool canGrabStill() const;
    virtual bool grabStill( QPixmap *pix );

    virtual bool needsNorm() const;

    virtual QWidget *configWidget(QWidget*, const char *);
    virtual void saveConfig();

private:
    QWidget *_w;
    QWidget *_dtReg;
    QWidget *_winReg;
    int _wReg, _hReg;
    V4LDev *dev;
    bool _useOverlay;
    V4LGrabber *g;
    QMutex _devMtx;
    QMap<QString,QString> _devNames;
    bool _probed;
    QVideoStream *_vs;
    QString _currentDev;
    int _norm;
    Clipper *_clipper;
    bool _capturing;
    V4LPluginCfg *_cfgWidget;
    int _qvsMethod;
    int _gsn;  // grabber sequence number
    bool _dvbs;

    int v4lSetup();

    bool _doRefresh;
    QPoint _refresh_ul;
    QSize _refresh_sz;
    
private slots:
    void triggerRepaint();
    void updateClipping();
    void repaintRegion(QPoint ul, QSize sz);
};


class Clipper : public QWidget {
Q_OBJECT
public:
    Clipper();
    ~Clipper();

    void setWidget(QWidget *w);
signals:
    void updateClipping();

protected:
    bool x11Event(XEvent *e);
    QWidget *_w;
};


#endif

