// -*- c++ -*-

/*
 *
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef QTVISIONVIEWIFACE_H
#define QTVISIONVIEWIFACE_H

#include <dcopobject.h>
#include <dcopref.h>

/**
 * A DCOP interface for QtVisionWidget.
 *
 * @author Richard Moore, rich@kde.org
 */
class QtVisionViewIface : virtual public DCOPObject
{
    K_DCOP
k_dcop:

    /**
     * Returns a DCOP reference to the QtVision object for the view.
     */
    virtual DCOPRef driverIface() const=0;

    /**
     * Returns the current view mode.
     */
    virtual int viewMode() const=0;

    /**
     * Returns true if the view mode is ViewNormal.
     */
    virtual bool isShowNormal() const=0;

    /**
     * Returns true if the view mode is ViewTopLevel.
     */
    virtual bool isShowTopLevel() const=0;

    /**
     * Returns true if the view mode is ViewFullScreen.
     */
    virtual bool isShowFullScreen() const=0;

    /**
     * Returns true if the view mode is ViewVideoDesktop.
     */
    virtual bool isShowVideoDesktop() const=0;

    /**
     * Returns true if the view mode is ViewNone.
     */
    virtual bool isShowNone() const=0;

    /**
     * Returns true if the screen widget has been detached from its
     * parent as a floating window.
     */
    virtual bool isFloating() const=0;

    /**
     * Forces a return to normal mode.
     */
    virtual void resetViewMode()=0;

    /**
     * Switches to the next view mode.
     */
    virtual void nextViewMode()=0;

    /**
     * Activates the specified view mode.
     */
    virtual void setViewMode( int mode )=0;

    virtual void setShowNone( bool active )=0;
    virtual void setShowNormal( bool active )=0;
    virtual void setShowTopLevel( bool active )=0;
    virtual void setShowFullScreen( bool active )=0;
    virtual void setShowVideoDesktop( bool active )=0;

    virtual void setShowNone()         { setShowNone(true);         };
    virtual void setShowNormal()       { setShowNormal(true);       };
    virtual void setShowTopLevel()     { setShowTopLevel(true);     };
    virtual void setShowFullScreen()   { setShowFullScreen(true);   };
    virtual void setShowVideoDesktop() { setShowVideoDesktop(true); };

    virtual void toggleFullScreen()=0;
};

#endif // QTVISIONVIEWIFACE_H


// Local Variables:
// c-basic-offset: 4
// End:

