/*
 *
 * Copyright (C) 2003 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "qvchannelplugin.h"
#include "channelstore.h"

#include <qfile.h>

#include <kdebug.h>
#include <kparts/componentfactory.h>


QVChannelPlugin::QVChannelPlugin(QtVision *qtv, const QString& cfgkey, QObject *o, const char* n)
: QVPluginBase(qtv, cfgkey, o, n), _flags(FormatReadWrite) {
}


QVChannelPlugin::~QVChannelPlugin() {
}


bool QVChannelPlugin::canRead(const QString& fmt) const {
	return (_flags & FormatRead) && (_fmtName == fmt);
}


bool QVChannelPlugin::canWrite(const QString& fmt) const {
	return (_flags & FormatWrite) && (_fmtName == fmt);
}


bool QVChannelPlugin::handlesFile(const QString& filename, int rflags) const {
	return ((_flags & rflags) == rflags) && filename.endsWith(_fmtName);
}


bool QVChannelPlugin::load(ChannelStore *store, const QString &filename, const QString& fmt) {
	QFile file(filename);
	file.open(IO_ReadWrite);

	if (!load(store, &file, fmt)) {
		kdDebug() << "QVChannelPlugin::load(...) sub-class load method failed" << endl;
		return false;
	}

	kdDebug() << "QVChannelPlugin::load(...) sub-class load ok" << endl;
	file.close();
	return true;
}


bool QVChannelPlugin::save(ChannelStore *store, const QString &filename, const QString& fmt) {
	QFile file(filename);
	file.open(IO_WriteOnly);

	if (!save(store, &file, fmt)) {
		kdDebug() << "QVChannelPlugin::save(...) sub-class save method failed" << endl;
		return false;
	}

	kdDebug() << "QVChannelPlugin::save(...) sub-class save ok" << endl;
	file.close();
	return true;
}


QVChannelPlugin::ChannelFileMetaInfo QVChannelPlugin::getMetaInfo(const QString &filename, const QString& fmt) {
	QFile file(filename);
	file.open(IO_ReadOnly);

	QVChannelPlugin::ChannelFileMetaInfo x = getMetaInfo(&file, fmt);

	file.close();
	return x;
}


QVChannelPlugin::ChannelFileMetaInfo QVChannelPlugin::getMetaInfo(QIODevice* /*file*/, const QString& /*fmt*/ ) {
	return QVChannelPlugin::ChannelFileMetaInfo();
}


bool QVChannelPlugin::load(ChannelStore *, QIODevice *, const QString&) {
	return false;
}

bool QVChannelPlugin::save(ChannelStore *, QIODevice *, const QString&) {
	return false;
}


#include "qvchannelplugin.moc"

