// -*- c++ -*-

/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */



#ifndef __QVPLUGINBASE_H
#define __QVPLUGINBASE_H

#include <qobject.h>
#include <qstring.h>
#include "pluginfactory.h"

class QVPluginBasePrivate;
class KConfig;
class QtVision;
class KActionCollection;
class KXMLGUIFactory;

/**
 * Plugin base class
 */
class QVPluginBase : public QObject
{
    Q_OBJECT

friend class PluginFactory;

public:
    QVPluginBase(QtVision *qtv, const QString& cfgkey, QObject *parent = 0, const char* name = 0);
    virtual ~QVPluginBase();

    const PluginDesc& pluginDescription() const;

    /**
     * Updates description. Should be called after reconfiguration providing the plugin desc
     * which matches pluginDescription() of object.
     */
    void updateDescription(const PluginDesc& desc) { _description = desc; }

    /**
     * Plugins can override this if they want to install GUI elements to the qtvision
     * client.
     * This method will be called when all the gui is properly initialized.
     */
    virtual void installGUIElements(KXMLGUIFactory * /*guiFactory*/,
	    KActionCollection * /*actionCollection*/) { }

    QtVision *driver() const;

public slots:
    virtual QWidget* configWidget(QWidget *parent, const char *name);
    virtual void saveConfig();
    
protected:
    PluginDesc _description;
    KConfig *_cfg;

private:
    QtVision *_qtv;
    QVPluginBasePrivate *d;
};

#endif

