#include <qtimer.h>
#include <kdebug.h>

#define fatal my_fatal

extern "C" {
#include "cache.h"
#include "vbi.h"
#include "fdset.h"
}

int alevt_debug = 0;

#include "teletextsource.h"
#include "teletextsource.moc"


TeletextSource::TeletextSource( QObject *parent, const char *name )
    : QObject( parent, name ),
      vbi(0), /*revealed(false), hold(false),*/ pgno(0x100), subno(0), vtp(0)
{
    fdset_init(fds);

    QTimer *t = new QTimer( this );
    connect( t, SIGNAL(timeout()), this, SLOT(hack()) );
    t->start(20);
}

TeletextSource::~TeletextSource()
{
}

bool TeletextSource::start( const char *dev )
{
    kdDebug() << "TeletextSource::start(): Opening device " << dev << endl;

    vbi = vbi_open( (char *) dev, cache_open(), 1/*fine_tune*/, -1/*newbttv*/ );
    if ( !vbi ) return false;

    if (vbi->cache)
	vbi->cache->op->mode(vbi->cache, CACHE_MODE_ERC, 1/*erc*/);

    vbi_add_handler( vbi, (void*)TeletextSource::dispatch_event, this );
    return true;
}

void TeletextSource::stop()
{
    vbi_close( vbi );
    vbi = 0;
}

void TeletextSource::setPage( int pgno )
{
    setPage( pgno, -1 );
}

void TeletextSource::setPage( int pgno, int subno )
{
    kdDebug() << "TeletextSource::setPage( " << pgno << ", " << subno << " )" << endl;

    if ( subno < 0 )
	subno = ANY_SUB;

//    if ( vbi_query_page( vbi, pgno, subno ) )
//	return;
    vbi_query_page( vbi, pgno, subno );

    this->pgno = pgno;
    this->subno = subno;
    vtp = 0;
}

void TeletextSource::setPage( TeletextPage *page )
{
    this->vtp = page;
    emit foundPage();
    emit foundPage( pgno );
    emit pageChanged( page );
}

void TeletextSource::dispatch_event( TeletextSource *tts, struct vt_event *ev )
{
//    kdDebug() << "teletextwidget_event(): Dispatching event" << endl;
    tts->teletextEvent( ev );
}

void TeletextSource::teletextEvent( TeletextEvent *ev )
{
//    kdDebug() << "TeletextSource::teletextEvent(): Event type " << ev->type << endl;

    switch ( ev->type ) {
	case EV_PAGE:
	{
	    TeletextPage *vtp = (TeletextPage *) ev->p1;

	    if ( vtp->pgno == pgno )
		setPage( vtp );
	    break;
	}
	case EV_HEADER:
	{
	    const char *p = (const char *) ev->p1;
	    int hdr_mag = ev->i1 / 256;
	    int flags = ev->i3;
	    int mag = pgno;

	    if (mag >= 0x10)
		mag = mag >> 4;
	    if (mag >= 0x10)
		mag = mag >> 4;

//	    kdDebug() << "p: " << p << endl;
//	    kdDebug() << "hdr_mag: " << hdr_mag << endl;
//	    kdDebug() << "flags: " << flags << endl;
//	    kdDebug() << "mag: " << mag << endl;
	    emit headerChanged( p );
	    break;
	}
	case EV_XPACKET:
	    break;
	default:
	    kdDebug() << "TeletextSource::teletextEvent(): Unknown event type" << endl;
	    break;
    }
}

void TeletextSource::hack()
{
    fdset_select(fds, -1);		// call scheduler
}
